/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>
#include <X11/cursorfont.h>  /* Necessary for creating a cursor of "?" mark */

static Widget toplevel;
static Widget dialog;
static Cursor question;

/* Callback function called when items on "File" menu is selected */
static void FileMenuCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    int menu_item = (int)client_data;  /* Item number of the selected menu */
    printf("File Menu: item %d is selected\n", menu_item);

    if (menu_item == 3)  /* If the selected item was 3 ("Exit"), */
        exit(0);         /* exit this program */
}

/* Callback function called when items on "Help" menu is selected */
static void HelpMenuCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget   context;
    XEvent   event;
    char     string[BUFSIZ];
    XmString message;

    int menu_item = (int)client_data;  /* Item number of the selected menu */
    printf("Help Menu: item %d is selected\n", menu_item);

    if (menu_item == 0) {  /* menu item 0 is "Context" Help */
        /* Grab event and allow user to select widget */
        context = XmTrackingEvent(toplevel, question, False, &event);
        if (context == NULL)               /* If the selected location is bad */
            XBell(XtDisplay(toplevel), 0);  /* notify by bell */
        else {
            /* Create a string for help message */
            sprintf(string, "Help message for widget `%s'", XtName(context));
            /* convert the string to compound string */
            message = XmStringCreateLocalized(string);
            /* Set the message to dialog */
            XtVaSetValues(dialog, XmNmessageString, message, NULL);
            XmStringFree(message); /* Free the unnecessary compound string */
            XtManageChild(dialog); /* Show the dialog */
        }
    }
}

/* Copy the selected text into clipboard */
static void CopyCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget text = (Widget)client_data;
    XmTextCopy(text, XtLastTimestampProcessed(XtDisplay(text)));
}

/* Cut the selected text into clipboard */
static void CutCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget text = (Widget)client_data;
    XmTextCut(text, XtLastTimestampProcessed(XtDisplay(text)));
}

/* Paste the data in clipboard into text */
static void PasteCB(w, client_data, call_data)
    Widget    w;
    XtPointer client_data;
    XtPointer call_data;
{
    Widget text = (Widget)client_data;
    XmTextPaste(text);
}

/* Create menubar and pulldown menu */
static Widget CreateMenu(parent)
    Widget parent;
{
    Widget  menubar, filemenu, helpbutton, helpmenu;
    Arg     al[20];
    int     ac;
    int     n = 0;

    /* Create menubar */
    menubar = XmVaCreateSimpleMenuBar(parent, "menubar",
                XmVaCASCADEBUTTON, NULL, NULL,  /* "File" menu */
                NULL);
    XtManageChild(menubar);

    /* Create "File" pulldown menu */
    filemenu = XmVaCreateSimplePulldownMenu(menubar, "filemenu",
                  n++, FileMenuCB,
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL,  /* "New" */
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL,  /* "Open" */
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL,  /* "Save" */
                  XmVaSEPARATOR,  /* Separator */
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL,  /* "Exit" */
                  NULL);

    /* Create button for "Help" menu */
    ac = 0;
    helpbutton = XmCreateCascadeButtonGadget(menubar, "helpbutton", al, ac);
    XtManageChild(helpbutton);

    /* Put the "Help" menu at the right of the menubar */
    XtVaSetValues(menubar, XmNmenuHelpWidget, helpbutton, NULL);

    helpmenu = XmVaCreateSimplePulldownMenu(menubar, "helpmenu",
                  n++, HelpMenuCB,
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL, /* "Context" */
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL, /* "Overview" */
                  XmVaPUSHBUTTON, NULL, NULL, NULL, NULL, /* "Version" */
                  NULL);

    return(menubar);
}

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget  mainwin, menubar, work, text, command, copy, cut, paste;
    Arg     al[20];
    int     ac;

    XtSetLanguageProc(NULL, NULL, NULL);

    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Contexthelp", NULL, 0,
                               &argc, argv, NULL, al, ac);

    ac = 0;
    dialog = XmCreateInformationDialog(toplevel, "dialog", al, ac);
    XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_CANCEL_BUTTON));
    XtUnmanageChild(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON));

    ac = 0;
    XtSetArg(al[ac], XmNshowSeparator, True); ac++;
    XtSetArg(al[ac], XmNcommandWindowLocation, XmCOMMAND_BELOW_WORKSPACE); ac++;
    mainwin = XmCreateMainWindow(toplevel, "mainwin", al, ac);
    XtManageChild(mainwin);

    menubar = CreateMenu(mainwin);   /* Create menu */

    ac = 0;
    work = XmCreateRowColumn(mainwin, "work", al, ac);
    XtManageChild(work);

    ac = 0;
    command = XmCreateRowColumn(mainwin, "command", al, ac);
    XtManageChild(command);

    XmMainWindowSetAreas(mainwin, menubar, command, NULL, NULL, work);

    ac = 0;
    text = XmCreateText(work, "text", al, ac);
    XtManageChild(text);

    ac = 0;
    copy = XmCreatePushButton(command, "copy", al, ac);
    XtManageChild(copy);
    ac = 0;
    cut = XmCreatePushButton(command, "cut", al, ac);
    XtManageChild(cut);
    ac = 0;
    paste = XmCreatePushButton(command, "paste", al, ac);
    XtManageChild(paste);

    XtAddCallback(copy,  XmNactivateCallback, CopyCB,  text);
    XtAddCallback(cut,   XmNactivateCallback, CutCB,   text);
    XtAddCallback(paste, XmNactivateCallback, PasteCB, text);

    /* Install accelerators on text widget */
    XtInstallAccelerators(text, copy);
    XtInstallAccelerators(text, cut);
    XtInstallAccelerators(text, paste);

    XtRealizeWidget(toplevel);

    /* Create "?" cursor */
    question = XCreateFontCursor(XtDisplay(toplevel), XC_question_arrow);

    XtAppMainLoop(app_context);
}
